# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import FileSystem
    from ._models_py3 import FileSystemList
    from ._models_py3 import LeaseAccessConditions
    from ._models_py3 import ModifiedAccessConditions
    from ._models_py3 import Path
    from ._models_py3 import PathHTTPHeaders
    from ._models_py3 import PathList
    from ._models_py3 import SourceModifiedAccessConditions
    from ._models_py3 import StorageError, StorageErrorException
    from ._models_py3 import StorageErrorError
except (SyntaxError, ImportError):
    from ._models import FileSystem
    from ._models import FileSystemList
    from ._models import LeaseAccessConditions
    from ._models import ModifiedAccessConditions
    from ._models import Path
    from ._models import PathHTTPHeaders
    from ._models import PathList
    from ._models import SourceModifiedAccessConditions
    from ._models import StorageError, StorageErrorException
    from ._models import StorageErrorError
from ._data_lake_storage_client_enums import (
    PathGetPropertiesAction,
    PathLeaseAction,
    PathRenameMode,
    PathResourceType,
    PathUpdateAction,
)

__all__ = [
    'FileSystem',
    'FileSystemList',
    'LeaseAccessConditions',
    'ModifiedAccessConditions',
    'Path',
    'PathHTTPHeaders',
    'PathList',
    'SourceModifiedAccessConditions',
    'StorageError', 'StorageErrorException',
    'StorageErrorError',
    'PathResourceType',
    'PathRenameMode',
    'PathUpdateAction',
    'PathLeaseAction',
    'PathGetPropertiesAction',
]
