# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AccessPolicy
    from ._models_py3 import ClearRange
    from ._models_py3 import CopyFileSmbInfo
    from ._models_py3 import CorsRule
    from ._models_py3 import DirectoryItem
    from ._models_py3 import FileHTTPHeaders
    from ._models_py3 import FileItem
    from ._models_py3 import FileProperty
    from ._models_py3 import FileRange
    from ._models_py3 import FilesAndDirectoriesListSegment
    from ._models_py3 import HandleItem
    from ._models_py3 import LeaseAccessConditions
    from ._models_py3 import ListFilesAndDirectoriesSegmentResponse
    from ._models_py3 import ListHandlesResponse
    from ._models_py3 import ListSharesResponse
    from ._models_py3 import Metrics
    from ._models_py3 import RetentionPolicy
    from ._models_py3 import ShareFileRangeList
    from ._models_py3 import ShareItemInternal
    from ._models_py3 import SharePermission
    from ._models_py3 import SharePropertiesInternal
    from ._models_py3 import ShareProtocolSettings
    from ._models_py3 import ShareSmbSettings
    from ._models_py3 import ShareStats
    from ._models_py3 import SignedIdentifier
    from ._models_py3 import SmbMultichannel
    from ._models_py3 import SourceModifiedAccessConditions
    from ._models_py3 import StorageError
    from ._models_py3 import StorageServiceProperties
except (SyntaxError, ImportError):
    from ._models import AccessPolicy  # type: ignore
    from ._models import ClearRange  # type: ignore
    from ._models import CopyFileSmbInfo  # type: ignore
    from ._models import CorsRule  # type: ignore
    from ._models import DirectoryItem  # type: ignore
    from ._models import FileHTTPHeaders  # type: ignore
    from ._models import FileItem  # type: ignore
    from ._models import FileProperty  # type: ignore
    from ._models import FileRange  # type: ignore
    from ._models import FilesAndDirectoriesListSegment  # type: ignore
    from ._models import HandleItem  # type: ignore
    from ._models import LeaseAccessConditions  # type: ignore
    from ._models import ListFilesAndDirectoriesSegmentResponse  # type: ignore
    from ._models import ListHandlesResponse  # type: ignore
    from ._models import ListSharesResponse  # type: ignore
    from ._models import Metrics  # type: ignore
    from ._models import RetentionPolicy  # type: ignore
    from ._models import ShareFileRangeList  # type: ignore
    from ._models import ShareItemInternal  # type: ignore
    from ._models import SharePermission  # type: ignore
    from ._models import SharePropertiesInternal  # type: ignore
    from ._models import ShareProtocolSettings  # type: ignore
    from ._models import ShareSmbSettings  # type: ignore
    from ._models import ShareStats  # type: ignore
    from ._models import SignedIdentifier  # type: ignore
    from ._models import SmbMultichannel  # type: ignore
    from ._models import SourceModifiedAccessConditions  # type: ignore
    from ._models import StorageError  # type: ignore
    from ._models import StorageServiceProperties  # type: ignore

from ._azure_file_storage_enums import (
    CopyStatusType,
    DeleteSnapshotsOptionType,
    FileRangeWriteType,
    LeaseDurationType,
    LeaseStateType,
    LeaseStatusType,
    ListSharesIncludeType,
    PermissionCopyModeType,
    ShareAccessTier,
    ShareRootSquash,
    StorageErrorCode,
)

__all__ = [
    'AccessPolicy',
    'ClearRange',
    'CopyFileSmbInfo',
    'CorsRule',
    'DirectoryItem',
    'FileHTTPHeaders',
    'FileItem',
    'FileProperty',
    'FileRange',
    'FilesAndDirectoriesListSegment',
    'HandleItem',
    'LeaseAccessConditions',
    'ListFilesAndDirectoriesSegmentResponse',
    'ListHandlesResponse',
    'ListSharesResponse',
    'Metrics',
    'RetentionPolicy',
    'ShareFileRangeList',
    'ShareItemInternal',
    'SharePermission',
    'SharePropertiesInternal',
    'ShareProtocolSettings',
    'ShareSmbSettings',
    'ShareStats',
    'SignedIdentifier',
    'SmbMultichannel',
    'SourceModifiedAccessConditions',
    'StorageError',
    'StorageServiceProperties',
    'CopyStatusType',
    'DeleteSnapshotsOptionType',
    'FileRangeWriteType',
    'LeaseDurationType',
    'LeaseStateType',
    'LeaseStatusType',
    'ListSharesIncludeType',
    'PermissionCopyModeType',
    'ShareAccessTier',
    'ShareRootSquash',
    'StorageErrorCode',
]
