# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AccessPolicy
    from ._models_py3 import CorsRule
    from ._models_py3 import DequeuedMessageItem
    from ._models_py3 import EnqueuedMessage
    from ._models_py3 import GeoReplication
    from ._models_py3 import ListQueuesSegmentResponse
    from ._models_py3 import Logging
    from ._models_py3 import Metrics
    from ._models_py3 import PeekedMessageItem
    from ._models_py3 import QueueItem
    from ._models_py3 import QueueMessage
    from ._models_py3 import RetentionPolicy
    from ._models_py3 import SignedIdentifier
    from ._models_py3 import StorageError
    from ._models_py3 import StorageServiceProperties
    from ._models_py3 import StorageServiceStats
except (SyntaxError, ImportError):
    from ._models import AccessPolicy  # type: ignore
    from ._models import CorsRule  # type: ignore
    from ._models import DequeuedMessageItem  # type: ignore
    from ._models import EnqueuedMessage  # type: ignore
    from ._models import GeoReplication  # type: ignore
    from ._models import ListQueuesSegmentResponse  # type: ignore
    from ._models import Logging  # type: ignore
    from ._models import Metrics  # type: ignore
    from ._models import PeekedMessageItem  # type: ignore
    from ._models import QueueItem  # type: ignore
    from ._models import QueueMessage  # type: ignore
    from ._models import RetentionPolicy  # type: ignore
    from ._models import SignedIdentifier  # type: ignore
    from ._models import StorageError  # type: ignore
    from ._models import StorageServiceProperties  # type: ignore
    from ._models import StorageServiceStats  # type: ignore

from ._azure_queue_storage_enums import (
    GeoReplicationStatusType,
    StorageErrorCode,
)

__all__ = [
    'AccessPolicy',
    'CorsRule',
    'DequeuedMessageItem',
    'EnqueuedMessage',
    'GeoReplication',
    'ListQueuesSegmentResponse',
    'Logging',
    'Metrics',
    'PeekedMessageItem',
    'QueueItem',
    'QueueMessage',
    'RetentionPolicy',
    'SignedIdentifier',
    'StorageError',
    'StorageServiceProperties',
    'StorageServiceStats',
    'GeoReplicationStatusType',
    'StorageErrorCode',
]
